@@ -32,11 +32,6 @@
  * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <errno.h>
-
 #include "k5-int.h"
 #include <kadm5/admin.h>
 #include <adm_proto.h>
@@ -52,7 +47,9 @@ static char *etype_string(krb5_enctype enctype);
 
 static int quiet;
 
+#ifdef KADMIN_LOCAL
 static int norandkey;
+#endif
 
 static void
 add_usage()
@@ -133,7 +130,9 @@ kadmin_keytab_add(int argc, char **argv)
 
     argc--; argv++;
     quiet = 0;
+#ifdef KADMIN_LOCAL
     norandkey = 0;
+#endif
     while (argc) {
         if (strncmp(*argv, "-k", 2) == 0) {
             argc--; argv++;
@@ -144,13 +143,9 @@ kadmin_keytab_add(int argc, char **argv)
             keytab_str = *argv;
         } else if (strcmp(*argv, "-q") == 0) {
             quiet++;
-        } else if (strcmp(*argv, "-norandkey") == 0) {
 #ifdef KADMIN_LOCAL
+        } else if (strcmp(*argv, "-norandkey") == 0) {
             norandkey++;
-#else
-            fprintf(stderr,
-                    _("-norandkey option only valid for kadmin.local\n"));
-            return;
 #endif
         } else if (strcmp(*argv, "-e") == 0) {
             argc--;
@@ -158,7 +153,7 @@ kadmin_keytab_add(int argc, char **argv)
                 add_usage();
                 return;
             }
-            retval = krb5_string_to_keysalts(*++argv, ", \t", ":.-", 0,
+            retval = krb5_string_to_keysalts(*++argv, NULL, NULL, 0,
                                              &ks_tuple, &n_ks_tuple);
             if (retval) {
                 com_err("ktadd", retval, _("while parsing keysalts %s"),
@@ -176,11 +171,13 @@ kadmin_keytab_add(int argc, char **argv)
         return;
     }
 
+#ifdef KADMIN_LOCAL
     if (norandkey && ks_tuple) {
         fprintf(stderr,
                 _("cannot specify keysaltlist when not changing key\n"));
         return;
     }
+#endif
 
     if (process_keytab(context, &keytab_str, &keytab))
         return;
@@ -287,11 +284,8 @@ add_principal(void *lhandle, char *keytab_str, krb5_keytab keytab,
         code = kadm5_get_principal_keys(handle, princ, &keys, &nkeys);
     else
 #endif
-        if (keepold || ks_tuple != NULL) {
-            code = kadm5_randkey_principal_3(lhandle, princ, keepold,
-                                             n_ks_tuple, ks_tuple, &keys, &nkeys);
-        } else
-            code = kadm5_randkey_principal(lhandle, princ, &keys, &nkeys);
+        code = randkey_princ(lhandle, princ, keepold, n_ks_tuple, ks_tuple,
+                             &keys, &nkeys);
     if (code != 0) {
         if (code == KADM5_UNK_PRINC) {
             fprintf(stderr, _("%s: Principal %s does not exist.\n"),
