@@ -1,7 +1,6 @@
 /* -*- mode: c; c-basic-offset: 4; indent-tabs-mode: nil -*- */
+/* lib/krb5/keytab/kt_file.c */
 /*
- * lib/krb5/keytab/kt_file.c
- *
  * Copyright 1990,1991,1995,2007,2008 by the Massachusetts Institute of Technology.
  * All Rights Reserved.
  *
@@ -23,12 +22,39 @@
  * M.I.T. makes no representations about the suitability of
  * this software for any purpose.  It is provided "as is" without express
  * or implied warranty.
+ */
+/*
+ * Copyright (c) Hewlett-Packard Company 1991
+ * Released to the Massachusetts Institute of Technology for inclusion
+ * in the Kerberos source code distribution.
  *
+ * Copyright 1990,1991 by the Massachusetts Institute of Technology.
+ * All Rights Reserved.
+ *
+ * Export of this software from the United States of America may
+ *   require a specific license from the United States Government.
+ *   It is the responsibility of any person or organization contemplating
+ *   export to obtain such a license before exporting.
+ *
+ * WITHIN THAT CONSTRAINT, permission to use, copy, modify, and
+ * distribute this software and its documentation for any purpose and
+ * without fee is hereby granted, provided that the above copyright
+ * notice appear in all copies and that both that copyright notice and
+ * this permission notice appear in supporting documentation, and that
+ * the name of M.I.T. not be used in advertising or publicity pertaining
+ * to distribution of the software without specific, written prior
+ * permission.  Furthermore if you modify this software you must label
+ * your software as modified software and not distribute it in such a
+ * fashion that it might be confused with the original M.I.T. software.
+ * M.I.T. makes no representations about the suitability of
+ * this software for any purpose.  It is provided "as is" without express
+ * or implied warranty.
  */
 
 #ifndef LEAN_CLIENT
 
 #include "k5-int.h"
+#include "../os/os-proto.h"
 #include <stdio.h>
 
 /*
@@ -97,9 +123,6 @@ extern const struct _krb5_kt_ops krb5_ktf_writable_ops;
 static krb5_error_code KRB5_CALLCONV
 krb5_ktfile_resolve(krb5_context, const char *, krb5_keytab *);
 
-static krb5_error_code KRB5_CALLCONV
-krb5_ktfile_wresolve(krb5_context, const char *, krb5_keytab *);
-
 static krb5_error_code KRB5_CALLCONV
 krb5_ktfile_get_name(krb5_context, krb5_keytab, char *, unsigned int);
 
@@ -162,21 +185,21 @@ krb5_ktfileint_find_slot(krb5_context, krb5_keytab, krb5_int32 *,
  * initialized with file keytab routines.
  */
 
-static krb5_error_code
-ktfile_common_resolve(krb5_context context, const char *name,
-                      krb5_keytab *idptr, const struct _krb5_kt_ops *ops)
+static krb5_error_code KRB5_CALLCONV
+krb5_ktfile_resolve(krb5_context context, const char *name,
+                    krb5_keytab *id_out)
 {
     krb5_ktfile_data *data = NULL;
     krb5_error_code err = ENOMEM;
     krb5_keytab id;
 
-    *idptr = NULL;
+    *id_out = NULL;
 
     id = calloc(1, sizeof(*id));
     if (id == NULL)
         return ENOMEM;
 
-    id->ops = ops;
+    id->ops = &krb5_ktf_ops;
     data = calloc(1, sizeof(krb5_ktfile_data));
     if (data == NULL)
         goto cleanup;
@@ -195,7 +218,7 @@ ktfile_common_resolve(krb5_context context, const char *name,
 
     id->data = (krb5_pointer) data;
     id->magic = KV5M_KEYTAB;
-    *idptr = id;
+    *id_out = id;
     return 0;
 cleanup:
     if (data)
@@ -205,12 +228,6 @@ cleanup:
     return err;
 }
 
-static krb5_error_code KRB5_CALLCONV
-krb5_ktfile_resolve(krb5_context context, const char *name, krb5_keytab *id)
-{
-    return ktfile_common_resolve(context, name, id, &krb5_ktf_writable_ops);
-}
-
 
 /*
  * "Close" a file-based keytab and invalidate the id.  This means
@@ -255,9 +272,7 @@ krb5_ktfile_get_entry(krb5_context context, krb5_keytab id,
     int was_open;
     char *princname;
 
-    kerror = KTLOCK(id);
-    if (kerror)
-        return kerror;
+    KTLOCK(id);
 
     if (KTFILEP(id) != NULL) {
         was_open = 1;
@@ -379,8 +394,8 @@ krb5_ktfile_get_entry(krb5_context context, krb5_keytab id,
         else {
             kerror = KRB5_KT_NOTFOUND;
             if (krb5_unparse_name(context, principal, &princname) == 0) {
-                krb5_set_error_message(context, kerror, "No key table entry "
-                                       "found for %s", princname);
+                k5_setmsg(context, kerror,
+                          _("No key table entry found for %s"), princname);
                 free(princname);
             }
         }
@@ -434,9 +449,7 @@ krb5_ktfile_start_seq_get(krb5_context context, krb5_keytab id, krb5_kt_cursor *
     krb5_error_code retval;
     long *fileoff;
 
-    retval = KTLOCK(id);
-    if (retval)
-        return retval;
+    KTLOCK(id);
 
     if (KTITERS(id) == 0) {
         if ((retval = krb5_ktfileint_openr(context, id))) {
@@ -458,8 +471,7 @@ krb5_ktfile_start_seq_get(krb5_context context, krb5_keytab id, krb5_kt_cursor *
         /* Wrapped?!  */
         KTITERS(id)--;
         KTUNLOCK(id);
-        krb5_set_error_message(context, KRB5_KT_IOERR,
-                               "Too many keytab iterators active");
+        k5_setmsg(context, KRB5_KT_IOERR, "Too many keytab iterators active");
         return KRB5_KT_IOERR;   /* XXX */
     }
     KTUNLOCK(id);
@@ -478,9 +490,7 @@ krb5_ktfile_get_next(krb5_context context, krb5_keytab id, krb5_keytab_entry *en
     krb5_keytab_entry cur_entry;
     krb5_error_code kerror;
 
-    kerror = KTLOCK(id);
-    if (kerror)
-        return kerror;
+    KTLOCK(id);
     if (KTFILEP(id) == NULL) {
         KTUNLOCK(id);
         return KRB5_KT_IOERR;
@@ -509,9 +519,7 @@ krb5_ktfile_end_get(krb5_context context, krb5_keytab id, krb5_kt_cursor *cursor
     krb5_error_code kerror;
 
     free(*cursor);
-    kerror = KTLOCK(id);
-    if (kerror)
-        return kerror;
+    KTLOCK(id);
     KTITERS(id)--;
     if (KTFILEP(id) != NULL && KTITERS(id) == 0)
         kerror = krb5_ktfileint_close(context, id);
@@ -612,7 +620,7 @@ krb5_ktf_keytab_externalize(krb5_context kcontext, krb5_pointer arg, krb5_octet
     size_t              remain;
     krb5_ktfile_data    *ktdata;
     krb5_int32          file_is_open;
-    krb5_int64          file_pos;
+    int64_t             file_pos;
     char                *ktname;
     const char          *fnamep;
 
@@ -709,7 +717,7 @@ krb5_ktf_keytab_internalize(krb5_context kcontext, krb5_pointer *argp, krb5_octe
     char                *ktname = NULL;
     krb5_ktfile_data    *ktdata;
     krb5_int32          file_is_open;
-    krb5_int64          foff;
+    int64_t             foff;
 
     *argp = NULL;
     bp = *buffer;
@@ -737,8 +745,7 @@ krb5_ktf_keytab_internalize(krb5_context kcontext, krb5_pointer *argp, krb5_octe
     if (kret)
         goto cleanup;
 
-    if (keytab->ops != &krb5_ktf_writable_ops
-        && keytab->ops != &krb5_ktf_ops) {
+    if (keytab->ops != &krb5_ktf_ops) {
         kret = EINVAL;
         goto cleanup;
     }
@@ -790,17 +797,6 @@ cleanup:
     return kret;
 }
 
-/*
- * This is an implementation specific resolver.  It returns a keytab id
- * initialized with file keytab routines.
- */
-
-static krb5_error_code KRB5_CALLCONV
-krb5_ktfile_wresolve(krb5_context context, const char *name, krb5_keytab *id)
-{
-    return ktfile_common_resolve(context, name, id, &krb5_ktf_writable_ops);
-}
-
 
 /*
  * krb5_ktfile_add()
@@ -811,14 +807,12 @@ krb5_ktfile_add(krb5_context context, krb5_keytab id, krb5_keytab_entry *entry)
 {
     krb5_error_code retval;
 
-    retval = KTLOCK(id);
-    if (retval)
-        return retval;
+    KTLOCK(id);
     if (KTFILEP(id)) {
         /* Iterator(s) active -- no changes.  */
         KTUNLOCK(id);
-        krb5_set_error_message(context, KRB5_KT_IOERR,
-                               "Cannot change keytab with keytab iterators active");
+        k5_setmsg(context, KRB5_KT_IOERR,
+                  _("Cannot change keytab with keytab iterators active"));
         return KRB5_KT_IOERR;   /* XXX */
     }
     if ((retval = krb5_ktfileint_openw(context, id))) {
@@ -846,14 +840,12 @@ krb5_ktfile_remove(krb5_context context, krb5_keytab id, krb5_keytab_entry *entr
     krb5_error_code     kerror;
     krb5_int32          delete_point;
 
-    kerror = KTLOCK(id);
-    if (kerror)
-        return kerror;
+    KTLOCK(id);
     if (KTFILEP(id)) {
         /* Iterator(s) active -- no changes.  */
         KTUNLOCK(id);
-        krb5_set_error_message(context, KRB5_KT_IOERR,
-                               "Cannot change keytab with keytab iterators active");
+        k5_setmsg(context, KRB5_KT_IOERR,
+                  _("Cannot change keytab with keytab iterators active"));
         return KRB5_KT_IOERR;   /* XXX */
     }
 
@@ -916,19 +908,21 @@ const struct _krb5_kt_ops krb5_ktf_ops = {
     krb5_ktfile_start_seq_get,
     krb5_ktfile_get_next,
     krb5_ktfile_end_get,
-    0,
-    0,
+    krb5_ktfile_add,
+    krb5_ktfile_remove,
     &krb5_ktfile_ser_entry
 };
 
 /*
- * krb5_ktf_writable_ops
+ * krb5_ktf_writable_ops -- this is the same as krb5_ktf_ops except for the
+ * prefix.  WRFILE should no longer be needed, but is effectively aliased to
+ * FILE for compatibility.
  */
 
 const struct _krb5_kt_ops krb5_ktf_writable_ops = {
     0,
     "WRFILE",   /* Prefix -- this string should not appear anywhere else! */
-    krb5_ktfile_wresolve,
+    krb5_ktfile_resolve,
     krb5_ktfile_get_name,
     krb5_ktfile_close,
     krb5_ktfile_get_entry,
@@ -959,36 +953,9 @@ const krb5_kt_ops krb5_kt_dfl_ops = {
     &krb5_ktfile_ser_entry
 };
 
+/* Formerly lib/krb5/keytab/file/ktf_util.c */
+
 /*
- * lib/krb5/keytab/file/ktf_util.c
- *
- * Copyright (c) Hewlett-Packard Company 1991
- * Released to the Massachusetts Institute of Technology for inclusion
- * in the Kerberos source code distribution.
- *
- * Copyright 1990,1991 by the Massachusetts Institute of Technology.
- * All Rights Reserved.
- *
- * Export of this software from the United States of America may
- *   require a specific license from the United States Government.
- *   It is the responsibility of any person or organization contemplating
- *   export to obtain such a license before exporting.
- *
- * WITHIN THAT CONSTRAINT, permission to use, copy, modify, and
- * distribute this software and its documentation for any purpose and
- * without fee is hereby granted, provided that the above copyright
- * notice appear in all copies and that both that copyright notice and
- * this permission notice appear in supporting documentation, and that
- * the name of M.I.T. not be used in advertising or publicity pertaining
- * to distribution of the software without specific, written prior
- * permission.  Furthermore if you modify this software you must label
- * your software as modified software and not distribute it in such a
- * fashion that it might be confused with the original M.I.T. software.
- * M.I.T. makes no representations about the suitability of
- * this software for any purpose.  It is provided "as is" without express
- * or implied warranty.
- *
- *
  * This function contains utilities for the file based implementation of
  * the keytab.  There are no public functions in this file.
  *
@@ -1063,7 +1030,7 @@ krb5_ktfileint_open(krb5_context context, krb5_keytab id, int mode)
     if (!KTFILEP(id)) {
         if ((mode == KRB5_LOCKMODE_EXCLUSIVE) && (errno == ENOENT)) {
             /* try making it first time around */
-            krb5_create_secure_file(context, KTFILENAME(id));
+            k5_create_secure_file(context, KTFILENAME(id));
             errno = 0;
             KTFILEP(id) = fopen(KTFILENAME(id), fopen_mode_rbplus);
             if (!KTFILEP(id))
@@ -1076,9 +1043,8 @@ krb5_ktfileint_open(krb5_context context, krb5_keytab id, int mode)
                 /* XXX */
                 return EMFILE;
             case ENOENT:
-                krb5_set_error_message(context, ENOENT,
-                                       "Key table file '%s' not found",
-                                       KTFILENAME(id));
+                k5_setmsg(context, ENOENT,
+                          _("Key table file '%s' not found"), KTFILENAME(id));
                 return ENOENT;
             default:
                 return errno;
@@ -1203,7 +1169,7 @@ krb5_ktfileint_delete_entry(krb5_context context, krb5_keytab id, krb5_int32 del
             }
         }
 
-        return krb5_sync_disk_file(context, KTFILEP(id));
+        return k5_sync_disk_file(context, KTFILEP(id));
     }
 
     return 0;
@@ -1296,7 +1262,7 @@ krb5_ktfileint_internal_read_entry(krb5_context context, krb5_keytab id, krb5_ke
     }
     u_princ_size = princ_size;
 
-    krb5_princ_set_realm_length(context, ret_entry->principal, u_princ_size);
+    ret_entry->principal->realm.length = u_princ_size;
     tmpdata = malloc(u_princ_size+1);
     if (!tmpdata) {
         error = ENOMEM;
@@ -1310,10 +1276,10 @@ krb5_ktfileint_internal_read_entry(krb5_context context, krb5_keytab id, krb5_ke
     tmpdata[princ_size] = 0;    /* Some things might be expecting null */
                                 /* termination...  ``Be conservative in */
                                 /* what you send out'' */
-    krb5_princ_set_realm_data(context, ret_entry->principal, tmpdata);
+    ret_entry->principal->realm.data = tmpdata;
 
     for (i = 0; i < count; i++) {
-        princ = krb5_princ_component(context, ret_entry->principal, i);
+        princ = &ret_entry->principal->data[i];
         if (!fread(&princ_size, sizeof(princ_size), 1, KTFILEP(id))) {
             error = KRB5_KT_END;
             goto fail;
@@ -1412,11 +1378,8 @@ krb5_ktfileint_internal_read_entry(krb5_context context, krb5_keytab id, krb5_ke
     return 0;
 fail:
 
-    for (i = 0; i < krb5_princ_size(context, ret_entry->principal); i++) {
-        princ = krb5_princ_component(context, ret_entry->principal, i);
-        if (princ->data)
-            free(princ->data);
-    }
+    for (i = 0; i < ret_entry->principal->length; i++)
+        free(ret_entry->principal->data[i].data);
     free(ret_entry->principal->data);
     ret_entry->principal->data = 0;
     free(ret_entry->principal);
@@ -1461,29 +1424,29 @@ krb5_ktfileint_write_entry(krb5_context context, krb5_keytab id, krb5_keytab_ent
     }
 
     if (KTVERSION(id) == KRB5_KT_VNO_1) {
-        count = (krb5_int16) krb5_princ_size(context, entry->principal) + 1;
+        count = (krb5_int16)entry->principal->length + 1;
     } else {
-        count = htons((u_short) krb5_princ_size(context, entry->principal));
+        count = htons((u_short)entry->principal->length);
     }
 
     if (!fwrite(&count, sizeof(count), 1, KTFILEP(id))) {
     abend:
         return KRB5_KT_IOERR;
     }
-    size = krb5_princ_realm(context, entry->principal)->length;
+    size = entry->principal->realm.length;
     if (KTVERSION(id) != KRB5_KT_VNO_1)
         size = htons(size);
     if (!fwrite(&size, sizeof(size), 1, KTFILEP(id))) {
         goto abend;
     }
-    if (!fwrite(krb5_princ_realm(context, entry->principal)->data, sizeof(char),
-                krb5_princ_realm(context, entry->principal)->length, KTFILEP(id))) {
+    if (!fwrite(entry->principal->realm.data, sizeof(char),
+                entry->principal->realm.length, KTFILEP(id))) {
         goto abend;
     }
 
-    count = (krb5_int16) krb5_princ_size(context, entry->principal);
+    count = (krb5_int16)entry->principal->length;
     for (i = 0; i < count; i++) {
-        princ = krb5_princ_component(context, entry->principal, i);
+        princ = &entry->principal->data[i];
         size = princ->length;
         if (KTVERSION(id) != KRB5_KT_VNO_1)
             size = htons(size);
@@ -1499,7 +1462,7 @@ krb5_ktfileint_write_entry(krb5_context context, krb5_keytab id, krb5_keytab_ent
      * Write out the principal type
      */
     if (KTVERSION(id) != KRB5_KT_VNO_1) {
-        princ_type = htonl(krb5_princ_type(context, entry->principal));
+        princ_type = htonl(entry->principal->type);
         if (!fwrite(&princ_type, sizeof(princ_type), 1, KTFILEP(id))) {
             goto abend;
         }
@@ -1548,7 +1511,7 @@ krb5_ktfileint_write_entry(krb5_context context, krb5_keytab id, krb5_keytab_ent
     if (fflush(KTFILEP(id)))
         goto abend;
 
-    retval = krb5_sync_disk_file(context, KTFILEP(id));
+    retval = k5_sync_disk_file(context, KTFILEP(id));
 
     if (retval) {
         return retval;
@@ -1564,7 +1527,7 @@ krb5_ktfileint_write_entry(krb5_context context, krb5_keytab id, krb5_keytab_ent
     }
     if (fflush(KTFILEP(id)))
         goto abend;
-    retval = krb5_sync_disk_file(context, KTFILEP(id));
+    retval = k5_sync_disk_file(context, KTFILEP(id));
 
     return retval;
 }
@@ -1580,15 +1543,13 @@ krb5_ktfileint_size_entry(krb5_context context, krb5_keytab_entry *entry, krb5_i
     krb5_int32 total_size, i;
     krb5_error_code retval = 0;
 
-    count = (krb5_int16) krb5_princ_size(context, entry->principal);
+    count = (krb5_int16)entry->principal->length;
 
     total_size = sizeof(count);
-    total_size += krb5_princ_realm(context, entry->principal)->length + (sizeof(krb5_int16));
+    total_size += entry->principal->realm.length + sizeof(krb5_int16);
 
-    for (i = 0; i < count; i++) {
-        total_size += krb5_princ_component(context, entry->principal,i)->length
-            + (sizeof(krb5_int16));
-    }
+    for (i = 0; i < count; i++)
+        total_size += entry->principal->data[i].length + sizeof(krb5_int16);
 
     total_size += sizeof(entry->principal->type);
     total_size += sizeof(entry->timestamp);
