@@ -50,8 +50,8 @@
  * organization.
  */
 #define KRB5_MAJOR_RELEASE 1
-#define KRB5_MINOR_RELEASE 12
-#define KRB5_PATCHLEVEL 4
+#define KRB5_MINOR_RELEASE 13
+#define KRB5_PATCHLEVEL 0
 /* #undef KRB5_RELTAIL */
-#define KRB5_RELDATE "20150529"
-#define KRB5_RELTAG "krb5-1.12.4-final"
+#define KRB5_RELDATE "20141015"
+#define KRB5_RELTAG "krb5-1.13-final"
